package net.minecraft.network.play.server;

import java.io.IOException;
import java.util.List;
import java.util.UUID;

import proxy.network.NetHandlerPlayServer;
import proxy.network.Packet;
import proxy.network.PacketBuffer;
import proxy.util.DataWatcher;
import proxy.util.MathHelper;

public class S0CPacketSpawnPlayer implements Packet<NetHandlerPlayServer>
{
    private int entityId;
    private UUID playerId;
    private int x;
    private int y;
    private int z;
    private byte yaw;
    private byte pitch;
    private int currentItem;
    private List<DataWatcher.WatchableObject> field_148958_j;

    public S0CPacketSpawnPlayer()
    {
    }

    public S0CPacketSpawnPlayer(int id, UUID uuid, double x, double y, double z, float yaw, float pitch)
    {
        this.entityId = id;
        this.playerId = uuid;
        this.x = MathHelper.floor_double(x * 32.0D);
        this.y = MathHelper.floor_double(y * 32.0D);
        this.z = MathHelper.floor_double(z * 32.0D);
        this.yaw = (byte)((int)(yaw * 256.0F / 360.0F));
        this.pitch = (byte)((int)(pitch * 256.0F / 360.0F));
        this.currentItem = 0;
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.entityId = buf.readVarIntFromBuffer();
        this.playerId = buf.readUuid();
        this.x = buf.readInt();
        this.y = buf.readInt();
        this.z = buf.readInt();
        this.yaw = buf.readByte();
        this.pitch = buf.readByte();
        this.currentItem = buf.readShort();
        this.field_148958_j = DataWatcher.readWatchedListFromPacketBuffer(buf);
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeVarIntToBuffer(this.entityId);
        buf.writeUuid(this.playerId);
        buf.writeInt(this.x);
        buf.writeInt(this.y);
        buf.writeInt(this.z);
        buf.writeByte(this.yaw);
        buf.writeByte(this.pitch);
        buf.writeShort(this.currentItem);
        DataWatcher.writeWatchedListToPacketBuffer(this.field_148958_j, buf);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(NetHandlerPlayServer handler)
    {
        handler.handleSpawnPlayer(this);
    }
}
